#!/usr/bin/env python
# -*- coding: utf8 -*-

# (c) 2017-2020 caregraf

import os
import sys
import re
import json
from collections import Counter
from datetime import datetime
from dateutil.relativedelta import relativedelta
import logging

from ..cacher.cacherUtils import FMQLReplyStore, FilteredResultIterator, BASE_LOCN_TEMPL, DATA_LOCN_TEMPL, DATARF_LOCN_TEMPL, metaOfVistA
from .reduceType import TypeReducer, SubTypeReducer, DEFAULT_FORCE_COUNT_PTER_TYPES, validDayDT
from .reportReductions import reportReductions
from .DEFAULT_TYPE_CONFIGS import TYPE_CONFIGS

TYPER_LOCN_TEMPL = BASE_LOCN_TEMPL + "Typer/"
TYPER_REDUCTIONS_LOCN_TEMPL = TYPER_LOCN_TEMPL + "Reductions/"
TYPER_REDUCTIONS_REPORTS_LOCN_TEMPL = TYPER_LOCN_TEMPL + "ReductionsReports/"
TYPER_REPORTS_LOCN_TEMPL = TYPER_LOCN_TEMPL + "Reports/"

"""
As command, fmqltyper

> fmqltyper {SNO} {TYP} {PERIODBACK From Cut} [overrideConfig file name]

So ex

> fmqltyper 442 2 YR1 myconfigFor2

where the override config is optional and there to allow typing evolve beyond
the per type settings in DEFAULT_TYPE_CONFIGS with ex/
{
        "subTypeProps": ["status ...
        "createDate": "... different create date"
etc.
}

Key items:
- onAndAfterDay/upToDay rely on create property and if set, want a "reductionLabel" to differentiate them. In the typical case, Period leads to settings for these.
- replyEfficiency (create property used to cut down reply walking) won't apply to flipped (RF) types (ex/ 44_003) or where create is not resource create (ex/ 409_84)
- 'countDateType' [YEAR|MONTH|WEEK|DAY] which decides the granularity of a date
counting. Usually its YEAR but shorter periods like DAY90 etc use DAY. This allows per
day activity tracking from type reductions.

TODO:
- try yaml config https://pyyaml.org/wiki/PyYAMLDocumentation as can embed code 
easily ie/ include custom filters/enhancers inline so no need for custom ie/ can't
now invoke dynamically in command line
"""
def reduceReportType(
    stationNumber, 
    typ, 
    period="ALL", 
    overrideConfig=None, 
    reductionLabelExplicit="", 
    customEnhancer=None
    ):
        
    # ###################### Make Configuration from sources ##########
        
    meta = metaOfVistA(stationNumber) # for cut date and annotations

    # Config is made to allow a loaded configuration to override all defaults
    config = {}
    
    if reductionLabelExplicit:
        config["reductionLabel"] = reductionLabelExplicit
    elif period != "ALL":
        config["reductionLabel"] = period
                    
    config["forceRedo"] = True
    
    """
    Map YR\d, DAY\d, MTH\d to date range, reductionLabel and default count date type
    """
    def enhanceConfigFromPeriod(config, vistaCutDate, period):

        if period == "ALL":
            config["countDateType"] = "YEAR"
            return
        
        vistaCutDay = vistaCutDate.split("T")[0]
        config["upToDay"] = vistaCutDay
    
        vistaCutDayDT = datetime.strptime(vistaCutDay, "%Y-%m-%d")
        lastDayDT = vistaCutDayDT - relativedelta(days=1)
        lastDay = datetime.strftime(lastDayDT, "%Y-%m-%d")
        
        relDelta = None
        for mtchRE, keyword, defaultCountDateType in [
            (r'YR(\d+)', "years", "YEAR"),
            (r'MTH(\d+)', "months", "MONTH"),
            (r'WK(\d+)', "weeks", "DAY"),
            (r'DAY(\d+)', "days", "DAY")
        ]:
            mtch = re.match(mtchRE, period)
            if not mtch:
                continue
            args = { keyword: int(mtch.group(1)) }
            relDelta = relativedelta(**args)
            onAndAfterDayDT = vistaCutDayDT - relDelta
            config["onAndAfterDay"] = datetime.strftime(onAndAfterDayDT, "%Y-%m-%d")
            config["countDateType"] = defaultCountDateType
            break
    
    # in config to allow overrides
    enhanceConfigFromPeriod(config, meta["cutDate"], period)

    if typ in TYPE_CONFIGS:
        config.update(TYPE_CONFIGS[typ])
    
    if overrideConfig: # as override allowed, must QA below
        config.update(overrideConfig) 
        
    config["dataLocnTempl"] = DATA_LOCN_TEMPL if not ("isRF" in config and config["isRF"]) else DATARF_LOCN_TEMPL
    if "replyWalkEfficiencyOff" not in config:
        config["replyWalkEfficiencyOff"] = True if "isRF" in config and config["isRF"] else False 
        
    # ####################### QA Settings ###############################
    # ... mainly needed for bugs in overrideConfig
        
    if "onAndAfterDay" in config:
        if not validDayDT(config["onAndAfterDay"]):
            raise Exception("Invalid 'on and after day': {}".format(config["onAndAfterDay"]))

    if "upToDay" in config:
        if not validDayDT(config["upToDay"]):
            raise Exception("Invalid 'up to day': {}".format(config["upToDay"]))
            
    if ("upToDay" in config or "onAndAfterDay" in config):
        if "reductionLabel" not in config:
            raise Exception("You must name the reduction ('reductionLabel') for time limited ('onAndAfterDay'/'upToDay') reductions")
        if "createDate" not in config:
            raise Exception("You must identify a 'create property' for time limited ('onAndAfterDay'/'upToDay') reductions")
            
    if "countDateType" in config and config["countDateType"] not in ["YEAR", "MONTH", "WEEK", "DAY"]:
        raise Exception("'countDateType' must be YEAR | MONTH | WEEK | DAY")
            
    # ################### Setup Env and See if Necessary ####################
                
    start = datetime.now()
    
    log = logging.getLogger('')
    log.setLevel(logging.DEBUG)    
    ch = logging.StreamHandler(sys.stdout)
    format = logging.Formatter("%(levelname)s - %(message)s")
    ch.setFormatter(format)
    log.addHandler(ch)
        
    # Used to ensure 
    def ensureLocation(locn):
        if not os.path.isdir(locn):
            os.mkdir(locn)
        return locn
    ensureLocation(TYPER_LOCN_TEMPL.format(stationNumber))
    reductionLocn = ensureLocation(TYPER_REDUCTIONS_LOCN_TEMPL.format(stationNumber))
    reportLocn = ensureLocation(TYPER_REPORTS_LOCN_TEMPL.format(stationNumber))        

    dataLocn = config["dataLocnTempl"].format(stationNumber)  
    redFFL = "{}/{}{}Reduction.json".format(reductionLocn, typ, config.get("reductionLabel", ""))
        
    if config["forceRedo"] == False and os.path.isfile(redFFL):
        logging.info("Not reducing (and reporting) {} as already exists and NOT forceredo".format(typ))
        return
        
    # ###################### Walk the Type and Reduce ####################
        
    if "reductionLabel" not in config:
        logging.info("About to reduce (and report) on {}, starting at {} [PID {}]".format(typ, start, os.getpid()))
    else:
        logging.info("About to reduce (and report) on {}, starting at {} [PID {}] - {} for onAndAfterDay {}, upToDay {}".format(typ, start, os.getpid(), config["reductionLabel"], config.get("onAndAfterDay", "-"), config.get("upToDay", "-"))) 
    
    fcpts = DEFAULT_FORCE_COUNT_PTER_TYPES
    if "forceCountPointerTypesExtra" in config:
        fcpts.extend(config["forceCountPointerTypesExtra"])
    if "subTypeProps" in config:
        logging.info("Subtyping by '{}'".format("-".join(config["subTypeProps"])))
        rtr = SubTypeReducer(
            typ, 
            createDateProp=config.get("createDate", ""), 
            subTypeProps=config["subTypeProps"], 
            forceCountPointerTypes=fcpts, 
            forceCountProperties=config.get("forceCountProperties", []), 
            countDateType=config.get("countDateType", "YEAR")
        )
    else:
        rtr = TypeReducer(
            typ, 
            createDateProp=config.get("createDate", ""), 
            forceCountPointerTypes=fcpts, 
            forceCountProperties=config.get("forceCountProperties", []), 
            countDateType=config.get("countDateType", "YEAR")
        )
        
    startAtReply = ""
    efficientWalked = False
    if "onAndAfterDay" in config:
        if config["replyWalkEfficiencyOff"]:
            logging.debug("No Start Efficiency: starting with first Reply and ignoring onAndAfterDay for skipping as either [1] walking reformed (RF) data which is usually from out of order multiples or [2] forced to walk all as property for onAndAfterDay is not create property (one in IEN order) ex/ appt start time or [3] no create date spec'ed")
        else:
            store = FMQLReplyStore(dataLocn)
            startAtReply = store.firstReplyFileOnOrAfterCreateDay(typ, config["createDate"], config["onAndAfterDay"])
            if startAtReply == "":
                logging.info("** Exiting: can't find Reply to start at on and after day {}, create property {}".format(onAndAfterDay, typesAndDates[typ]))
                return
            logging.debug("Start Efficiency: configuring Iterator for on and after time: on or after day {}, create property {}, starting at reply {}".format(config["onAndAfterDay"], config["createDate"], startAtReply))
            efficientWalked = True
    resourceIter = FilteredResultIterator(dataLocn, typ, startAtReply=startAtReply)
                                
    def getCreateDay(resource, createDateProp):
        value = resource[createDateProp]["value"] if "value" in resource[createDateProp] else resource[createDateProp]["label"]
        dayValue = value.split("T")[0] # using lex comp on ISO‑8601 form as fastest
        if not validDayDT(dayValue):
            log.debug("Can't get create day as its date has invalid value: {}".format(dayValue))
            return None
        return dayValue  
        
    msgThres = 50000 
    upToDayStopped = False
    count = 0 
    filtered = Counter()  
    for i, resource in enumerate(resourceIter, 1):
        count += 1
        filteredSoFar = sum(filtered[c] for c in filtered)
        if i % msgThres == 0:
                logging.debug("Checked {} more resources for total of {:,}, sub type reducers {:,}, so far filtered {:,}, now in reply {} - {}".format(msgThres, i, rtr.subTypeReducerCount(), filteredSoFar, resourceIter.currentReplyFile(), datetime.now() - start))
        idProp = "id" if "id" in resource else "_id"
        if customEnhancer:
            resource = customEnhancer.enhance(resource)
            if not resource:
                filtered["CUSTOM"] += 1
                continue
        # Allows for out of order too
        if "onAndAfterDay" in config or "upToDay" in config:
            if config["createDate"] not in resource:
                filtered["NOCREATEDATE"] += 1
                continue
            createDay = getCreateDay(resource, config["createDate"])
            if not createDay:
                filtered["BADCREATEDATE"] += 1
                continue
            if "onAndAfterDay" in config and createDay < config["onAndAfterDay"]:
                filtered["LTONAFTER"] += 1
                continue
            if "upToDay" in config and createDay >= config["upToDay"]:
                filtered["GTEUPTO"] += 1
                continue 
        rtr.transform(resource)
            
    pattData = rtr.reductions()
    # Add more meta to ALL Reduction
    allReds = [typeData for typeData in pattData if "_subTypeId" not in typeData]
    if len(allReds) != 1:
        raise Exception("Internal Error - expected one and only one ALL reduction")
    allRed = allReds[0]
    allRed["_stationNumber"] = stationNumber
    allRed["_vistaName"] = meta.get("name", "UNKNOWN")
    allRed["_vistaCut"] = meta["cutDate"]
    if efficientWalked:
        allRed["_efficientWalked"] = True
    allRed["_label"] = config.get("reductionLabel", "ALL")
    if "onAndAfterDay" in config:
        allRed["_onAndAfterDay"] = config["onAndAfterDay"]
    if "upToDay" in config:
        allRed["_upToDay"] = config["upToDay"]
                            
    json.dump(pattData, open(redFFL, "w")) 
     
    reportLocn = ensureLocation(TYPER_REPORTS_LOCN_TEMPL.format(stationNumber))
    reportReductions(pattData, "{}/{}{}Report.txt".format(reportLocn, typ, config.get("reductionLabel", "")))
        
    if len(filtered):
        filteredMU = "/".join(["{} [{}]".format(reason, filtered[reason]) for reason in filtered])
        logging.info("Finished after {:,} resources of {} in {} at {}, {:,} explicitly filtered with {}".format(count, typ, datetime.now() - start, datetime.now(), sum(filtered[c] for c in filtered), filteredMU))
    else:
        logging.info("Finished after {:,} resources of {} in {} at {}".format(count, typ, datetime.now() - start, datetime.now()))
        
# ############################# Driver ####################################

def main():

    assert sys.version_info >= (3, 4)
    
    try:
        stationNumber = sys.argv[1]
        typ = sys.argv[2]
    except IndexError:
        raise SystemExit("Usage _EXE_ STATIONNO TYP [PERIOD] [OVERRIDE CONFIG]")
    
    if not re.match(r'\d{3}$', stationNumber):
        raise SystemExit("Need three digit station number")
        
    if len(sys.argv) == 3:
        period = "ALL"
    else:
        period = sys.argv[3]
        if not re.match(r'(ALL|YR|MTH|WK|DAY)', period):
            raise SystemExit("Invalid period {}".format(period))
        
    # For this specific type - can override all but SNO and typ itself
    overrideConfig = None
    if len(sys.argv) > 4:
        overrideConfigName = sys.argv[4].split(".")[0] 
        if not os.path.isfile("{}.json".format(overrideConfigName)):
            raise SystemExit("No override config file {}.json - exiting".format(overrideConfigName))
        overrideConfig = json.load(open("{}.json".format(overrideConfigName)))
    
    reduceReportType(stationNumber, typ, period, overrideConfig=overrideConfig)
        
if __name__ == "__main__":
    main()
    



#!/usr/bin/env python
# -*- coding: utf8 -*-

# (c) 2019-2020 caregraf

import sys
import os
import json
from fmqlutils.cacher.cacher import makeFMQLIF
from fmqlutils.cacher.cacherUtils import FMQLReplyStore, DATA_LOCN_TEMPL, configLogging, recacheForCSTOPs

def cacheStopped(config, onlies=None):

    dmLocation = DATA_LOCN_TEMPL.format(stationNumber)
    fmqlReplyStore = FMQLReplyStore(dmLocation, makeDir=False)
    stoppedTypes = fmqlReplyStore.stoppedTypes()
    if onlies:
        print("Only recaching {}".format(
            ", ".join(sorted(list(onlies)))
        ))
        if len(set(onlies) - set(stoppedTypes)):
            print("Removed {} from list of 'only types' as not stopped".format(
                ", ".join(sorted(list(set(onlies) - set(stoppedTypes))))
            ))
            stoppedTypes = set(onlies).intersection(set(stoppedTypes))
    if len(stoppedTypes) == 0:
        raise SystemExit("No stopped types to recache - exiting") 
    cantRecacheTyps = {}
    recached = set()
    for typ in stoppedTypes:
        try:
            recacheForCSTOPs(config["stationNumber"], typ, makeFMQLIF(config))
        except Exception as e:
            cantRecacheTyps[typ] = e
            print("Can't recache {}".format(typ))
        else:
            recached.add(typ)
    print("Done with {:,} stopped types - {:,} succeeded , {:,} failed - {}".format(
        len(stoppedTyped),
        len(cantRecacheTyps),
        ", ".join(sorted(list(cantRecacheTyps)))
    ))
 
"""
> {EXE} SNO_Config -- all stopped types
or
> {EXE} SNO_Config TYP -- one stopped type
"""   
def main():

    assert sys.version_info >= (3, 4)
    
    try:
        sysConfigName = sys.argv[1].split(".")[0]
    except IndexError:
        raise SystemExit("Usage _EXE_ {SYSTEM CONFIG FILE} [{typ}]")
    
    if not os.path.isfile("{}.json".format(sysConfigName)):
        raise SystemExit("No system config file {}.json - exiting".format(sysConfigName))
    try:
        config = json.load(open("{}.json".format(sysConfigName)))
    except:
        raise SystemExit("Invalid system config {}.json - can't parse".format(sysConfigName))

    configLogging(config["stationNumber"], "cacherLog", config["logLevel"] if "logLevel" in config else "INFO")
    
    onlies = None
    if len(sys.argv) > 2:
        onlies = [sys.argv[2]]
    cacheStopped(config, onlies)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf8 -*-

"""
Per (Schema) Type settings for Type Reduction. These may be overridden.

Type Reduction also needs: {SNO}

Companion default 'constants' for typing: 
- period + reductionLabel = "ALL" and countDateType = YEAR
- forceRedo = True
- dataLocationTemplate: "/data/vista/{}/Data/", (as opposed to DataRF for isRF)
"""
TYPE_CONFIGS = {

    "2_98": {
    
        "subTypeProps": ["clinic", "status"],
        
        "createDate": "date_appt_made",
        
        "forceCountProperties": ["scheduling_request_type"]
        
    },
    
    # May add "forceExtension": True ie/ getting duration etc
    "3_081": {
    
        "subTypeProps": ["user", "level_of_assurance", "remote_app", "remote_200_user_ien", "workstation_label", "workstation_type", "remote_station_id", "device"],
            
        "createDate": "date_time",
        
        "forceCountProperties": ["ipv4_address", "remote_station_id", "remote_user_ien"]
        
    },

    "9_7": {

        "createDate": "date_loaded"

    },
    
    "19": {
    
        "subTypeProps": ["type-4"]
        
    },
    
    "26_13": {
    
        "subTypeProps": ["status", "flag_name", "owner_site"]
        
    },

    "26_14": {
            
        "createDate": "date_time",
        
        "subTypeProps": ["action", "entered_by"],
        
        "forceCountPointerTypesExtra": ["8925_1"]
        
    },
    
    "44": {
    
        "subTypeProps": ["stop_code_number", "type_extension", "division"]
        
    },
    
    "44_001": {
    
        "subTypeProps": ["multiple_parent"]
        
    },
    
    "44_003": {

        "createDate": "appointment_date_time",
    
        "isRF": True,
    
        "subTypeProps": ["appointment_container_", "#appointment_cancelled"],
        
        "replyWalkEfficiencyOff": True,

        "note": "efficient walk off as (ala 409_84) [1] using appt start time and not entry time as the create date and [2] this is a flipped multiple"
    
    },

    "45": {
        
        "createDate": "admission_date",
        
        "subTypeProps": ["status", "discharge_status"]
        
    },

    "52": {
            
        "createDate": "login_date",
        
        "subTypeProps": ["status"]
        
    },

    "63_04": {

        "isRF": True,

        "replyWalkEfficiencyOff": True,

        "note": "NOT COMPLETE"

    },

    "74": {
            
        "createDate": "date_report_entered"
        
    },

    "120_5": {
            
        "createDate": "date_time_vitals_taken",
        
        "subTypeProps": ["vital_type", "entered_in_error"]
        
    },

    "120_8": {
        
        "createDate": "origination_date_time",
        
        "subTypeProps": ["gmr_allergy", "entered_in_error"]
        
    },

    "120_86": {
            
        "createDate": "assessment_date_time"
        
    },

    "123": {
            
        "createDate": "file_entry_date",
        
        "subTypeProps": ["cprs_status", "routing_facility", "to_service"]
        
    },

    "221": {
        
        "createDate": "date"
        
    },

    "228_1": {
        
        "createDate": "date_created",

        "subTypeProps": ["location"]
        
    },

    "228_2": {
            
        "createDate": "date"
        
    },
    
    "405": {
    
        "subTypeProps": ["transaction"],
        
        "forceCountProperties": ["roombed", "ward_location"]
        
    },

    "409_68": {
            
        "createDate": "visit_file_entry",
        
        "subTypeProps": ["status"],
        
        "forceCountPointerTypesExtra": ["40_7"],
        
        "replyWalkEfficiencyOff": True
        
    },
    
    "409_831": {
    
        "subTypeProps": ["hospital_location"]
    
    },
    
    "409_833": {
    
        "subTypeProps": ["username"]
        
    },

    "409_84": {
        
        "createDate": "starttime",
        
        "subTypeProps": ["resource", "#cancel_datetime", "#noshow_datetime"],

        "replyWalkEfficiencyOff": True,

        "note": "efficient walk off as using appt start time and not entry time as the create date"
        
    },

    "627_8": {
            
        "createDate": "date_time_of_diagnosis",
        
        "forceCountProperties": ["axis_5"],
        
    },

    "2005": {
        
        "createDate": "date_time_image_saved",
        
        "subTypeProps": ["object_type", "capture_application", "dicom_sop_class", "spec_subspec_index", "status"],
        
        "forceCountProperties": ["group_parent"],
        
        "forceCountPointerTypesExtra": ["71", "2005_81", "2005_82", "2005_83", "2005_84", "2005_85", "2006_04", "2006_81", "8925_1"]
        
    },
    
    "2006_5849": {

        "createDate": "acquisition_start",

        "forceCountProperties": ["consult_audit_trail_transitions"],
    
        "subTypeProps": ["reader_duz_at_acquisition_site", "image_index_for_specialty", "consult_service", "status", "#reading_start"]
        
    },

    "2006_82": {
            
        "createDate": "start",
        
        "subTypeProps": ["start_mode", "workstation"],
        
        "forceCountProperties": ["name"],
        
        "forceCountPointerTypesExtra": ["2006_81"]
        
    },

    "8925": {
            
        "createDate": "entry_date_time",
    
        "subTypeProps": ["document_type", "status"],
        
        "forceCountProperties": ["parent__06_document_type_label"],
        
        "forceCountPointerTypesExtra": ["40_7"]
    
    },
    
    "8989_5": {
    
        "subTypeProps": ["entity"]
        
    },

    "9000010": {
            
        "createDate": "visit_admit_datetime",
        
        # Saw alternative of "service_category", "hospital_location"
        "subTypeProps": ["service_category", "dss_id"],
        
        "forceCountPointerTypesExtra": ["40_7"]
        
    },

    "9000011": {
            
        "createDate": "date_entered",

        "subTypeProps": ["provider_narrative", "condition", "status"],
        
        "forceCountProperties": ["diagnosis", "problem"]
        
    },

    "9000010_11": {
            
        "createDate": "visit",
        
        "subTypeProps": ["immunization", "data_source"]
        
    },

    "9000010_23": {
        
        "createDate": "visit",
        
        "subTypeProps": ["health_factor"],
        
        "forceCountPointerTypesExtra": ["9000010"]
        
    },
    
    "9999999_64": {
    
        "subTypeProps": ["entry_type"]
        
    },
    
    "9000010_99999": {
    
        "isRF": True,
        
        "subTypeProps": ["location"],
        
        "createDate": "date_time_start"
        
    },

    "354_71": {

        "subTypeProps": ["status", "institution"],
        
        "createDate": "date_entry_added"

    }
}



#!/usr/bin/env python
# -*- coding: utf8 -*-

# (c) 2015-2020 caregraf

import os
import sys
import json
import shutil
from datetime import datetime
import re
from collections import defaultdict
import logging

from .cacherUtils import FMQLReplyStore, V1RecordToV2, ensureCacheLocations, DATAV1_LOCN_TEMPL, DATA_LOCN_TEMPL, LOG_LOCN_TEMPL

"""
Turn FMQL v1.* form replies to v2 form 
- supports both zip and "raw" JSON caches, for both source and target
  - REM: v1 is zip or json; v2 can be either, set with 'useZip'
- will restart where it left off or rebuild from scratch
- notes any transformation errors (shouldn't happen)
"""
def toV2Data(stationNumber, onlyTypes=None, forceRedo=False, useZip=True, forMongo=True):

    ensureCacheLocations(stationNumber)

    log = logging.getLogger('')
    log.setLevel(logging.DEBUG)    
    ch = logging.StreamHandler(sys.stdout)
    format = logging.Formatter("%(levelname)s - %(message)s")
    ch.setFormatter(format)
    log.addHandler(ch)

    systemId = stationNumber
    logging.info("V2ing Data of VistA {} [PID {}]".format(systemId, os.getpid()))
    
    v1ReplyStore = FMQLReplyStore(DATAV1_LOCN_TEMPL.format(stationNumber))
    logging.info("Started Reply Cache V1 - uses {} - total replies {}".format("ZIP" if v1ReplyStore.usesZip() else "JSON", v1ReplyStore.totalReplies()))
        
    v2ReplyStore = FMQLReplyStore(DATA_LOCN_TEMPL.format(stationNumber), useZip=useZip)
    logging.info("Started Reply Cache V2 - uses {} - total replies {}".format("ZIP" if v2ReplyStore.usesZip() else "JSON", v2ReplyStore.totalReplies()))
    
    start = datetime.now()
    logging.info("Starting transformation of V1 to V2 Data at {}".format(str(start)))

    # May be .zip even if v1 is json
    existingReplyFiles = set(v2ReplyStore.replyFiles())
    def excludeV2AlreadyReply(replyFile):
        if replyFile in existingReplyFiles:
            logging.info("{} Being Skipped as already V2 (no forceRedo)".format(replyFile))
            return True
        return False
        
    replyIterator = v1ReplyStore.iterator(onlyTypes=onlyTypes, excludeRepliesFilter=excludeV2AlreadyReply if not forceRedo else None)
    problems = []
    overallWStops = {}
    ortn = V1RecordToV2(forMongo=forMongo)
    transformed = 0
    transformedRecords = 0
    startT = datetime.now() # start before iteration
    for reply in replyIterator:
        
        nreply = {"results": []}
        for nrProp in reply:
            if nrProp in ["count", "results"]:
                continue
            nreply[nrProp] = reply[nrProp]
                
        for record in reply["results"]:
            try: 
                nrecord = ortn.transform(record)
                transformedRecords += 1
                nreply["results"].append(nrecord)
                if "fmqlHasStops" in nrecord:
                    if "fmqlHasStops" not in nreply:
                        nreply["fmqlHasStops"] = -1
                    if nreply["fmqlHasStops"] < nrecord["fmqlHasStops"]:
                        nreply["fmqlHasStops"] = nrecord["fmqlHasStops"]
            except Exception as e:
                logging.exception(e)
                logging.info("EXCEPTION transforming in {}".format(record["uri"]["value"]))
                problems.append(record["uri"]["value"])
                continue
        
        nreplyFile = v2ReplyStore.flush(nreply)
        # Want to know if results in reply embed stop information
        if "fmqlHasStops" in nreply:
            overallWStops[nreplyFile] = nreply["fmqlHasStops"]
        transformed += 1
        logging.info("{}. Transformed and Flushed {} in {} out of {}".format(transformed, nreplyFile, str(datetime.now() - startT), str(datetime.now() - start)))
        startT = datetime.now()
        
    timeTaken = str(datetime.now() - start)
    if len(overallWStops) == 0:
        logging.info("Ending transformation - {:,} records in {:,} replies, problem {}, NO STOPS, over {}".format(transformedRecords, transformed, len(problems), timeTaken))
    else:
        logging.info("Ending transformation - {:,} records in {:,} replies, problem {}, w/stop(s) {:,}, max stopped length {:,}, over {}".format(transformedRecords, transformed, len(problems), len(overallWStops), max(overallWStops[rf] for rf in overallWStops), timeTaken))

# ############################# Driver ####################################

def main():

    assert sys.version_info >= (3, 4)
    
    try:
        stationNumber = sys.argv[1]
        typ = sys.argv[2]
    except IndexError:
        raise SystemExit("Usage _EXE_ STATIONNO TYP [FORCEREDO=NO|YES]")

    forceRedo = True
    if len(sys.argv) > 3:
        forceRedoArg = sys.argv[3] 
        if forceRedoArg.lower() == "no":
            forceRedo = False
    
    toV2Data(stationNumber, onlyTypes=[typ], forceRedo=forceRedo, forMongo=True)

if __name__ == "__main__":
    main()



from setuptools import setup

setup(name='fmqlutils',
      description = 'FMQL Utilities',
      long_description = """A Python framework and set of executables for caching datasets from FileMan systems using FMQL. Includes support for data analysis and modeling""",
      version='3.5',
      install_requires = [
          'rpcutils>=3.0',
          'python-dateutil'
          # 'pytz>=2019.1'
      ],
      python_requires='>=3.4, <4',
      classifiers = ["Development Status :: 4 - Beta", "Programming Language :: Python :: 3"],
      url='http://github.com/Caregraf/fmqlutils',
      license='Apache License, Version 2.0',
      keywords='VistA,FileMan,CHCS,FMQL',
      package_dir = {'fmqlutils': ''},
      packages = ['fmqlutils', 'fmqlutils.cacher', 'fmqlutils.typer', 'fmqlutils.reporter'],
      entry_points = {
          'console_scripts': ['fmqlcacher=fmqlutils.cacher.cacher:main', 'fmqlv2er=fmqlutils.cacher.v2er:main', 'fmqlrecacher=fmqlutils.cacher.recacher:main', 'fmqlcachehealth=fmqlutils.reporter.cacheHealthReporter:main', 'fmqltyper=fmqlutils.typer.reduceReportType:main']
      }
)
